#! /usr/bin/perl
use strict;

my $pkgdir = shift(@ARGV);
my $outdir = shift(@ARGV);
my $target = shift(@ARGV);
my $community = shift(@ARGV);
my $local_name = shift(@ARGV);


$outdir="$outdir/$target";
my $tmpdir="$pkgdir/tmp/$target";

my %NAMES;
my $as_names = "$pkgdir/etc/as_names.txt";
open ( FILE, $as_names ) || die "could not open $as_names\n";
while (my $line = <FILE> )
{
	chomp($line);
	$line =~ s/^\s*//g;
	$line =~ s/\s*$//g;
	if ( $line =~ /^$/ || $line =~ /^#/ ) { next; }
	my ( $as, $name ) = split( " ", $line, 2 );
	$NAMES{ $as } = $name;
}
close(FILE);


my %AS;

#
# COLLECT OUR DATA
#
use coqm;
my $rh_session = coqm::create_session($target, $community );
my $rh_Peers = coqm::fetch_bgpPeerIdentifier();
my $rh_routes = coqm::fetch_bgp4PathAttrBest();

foreach my $peer_ip (sort { $a <=> $b } (keys %{$rh_Peers}) )
{
	my $as = coqm::fetch_bgpPeerRemoteAs( $peer_ip );
	my $state = coqm::fetch_bgpPeerState( $peer_ip );
	my $version = coqm::fetch_bgpPeerNegotiatedVersion( $peer_ip );
	my $laddr = coqm::fetch_bgpPeerLocalAddr( $peer_ip );
	my $uptime = coqm::fetch_bgpPeerFsmEstablishedTime( $peer_ip );
	my $InUpdates = coqm::fetch_bgpPeerInUpdates( $peer_ip );
	my $OutUpdates = coqm::fetch_bgpPeerOutUpdates( $peer_ip );
	my $TimeSinceLastUpdate = coqm::fetch_bgpPeerInUpdateElapsedTime( $peer_ip );
	my $InMessages = coqm::fetch_bgpPeerInTotalMessages( $peer_ip );
	my $OutMessages = coqm::fetch_bgpPeerOutTotalMessages( $peer_ip );
	my $ActiveKeepAlive = coqm::fetch_bgpPeerKeepAlive( $peer_ip );
	my $ConfKeepAlive = coqm::fetch_bgpPeerKeepAliveConfigured( $peer_ip );
	my $ActiveHoldTime = coqm::fetch_bgpPeerHoldTime( $peer_ip );
	my $ConfHoldTime = coqm::fetch_bgpPeerHoldTimeConfigured( $peer_ip );

	$AS{ $as }{ $peer_ip }{ 'state' } = $state;
	$AS{ $as }{ $peer_ip }{ 'version' } = $version;
	$AS{ $as }{ $peer_ip }{ 'uptime' } = time_to_text( $uptime );
	$AS{ $as }{ $peer_ip }{ 'laddr' } = $laddr;
	$AS{ $as }{ $peer_ip }{ 'InUpdates' } = $InUpdates;
	$AS{ $as }{ $peer_ip }{ 'OutUpdates' } = $OutUpdates;
	$AS{ $as }{ $peer_ip }{ 'InMessages' } = $InMessages;
	$AS{ $as }{ $peer_ip }{ 'OutMessages' } = $OutMessages;
	$AS{ $as }{ $peer_ip }{ 'TimeSinceLastUpdate' } = time_to_text( $TimeSinceLastUpdate );
	$AS{ $as }{ $peer_ip }{ 'ActiveKeepAlive' } = $ActiveKeepAlive;
	$AS{ $as }{ $peer_ip }{ 'ConfKeepAlive' } = $ConfKeepAlive;
	$AS{ $as }{ $peer_ip }{ 'ActiveHoldTime' } = $ActiveHoldTime;
	$AS{ $as }{ $peer_ip }{ 'ConfHoldTime' } = $ConfHoldTime;
	foreach my $network (sort (keys %{$$rh_routes{'by_peer'}{ $peer_ip }}))
	{
		foreach my $mask (sort (keys %{$$rh_routes{'by_peer'}{ $peer_ip }{ $network }}))
		{
			my $best = $$rh_routes{'by_peer'}{ $peer_ip }{ $network }{ $mask };
			$AS{ $as }{ $peer_ip }{ 'routes' }{ "$network/$mask" }{ 'best' } = $best;
		}
	}
			
}



#
# GENERATE OUR DATA AND HTML FILES
#
my $route_counts = "$tmpdir/route_counts.txt";
system( 'mv', $route_counts, "$route_counts.last" );

open( COUNTS, ">>$route_counts" );

my $main = "$outdir/index.html";
open( MAIN, ">$main" );

my $now = time();

select(MAIN);
print "<HTML>\n";
print "<TITLE>$local_name - $target BGP Peerings</TITLE>\n";
print "<CENTER>\n";
print "<H3>$local_name - $target BGP Peerings</H3>\n";
print "<TABLE WIDTH=\"100%\" BORDER=\"0\"><TR><TD>\n";

foreach my $as (sort { $a <=> $b } (keys %AS))
{
	print "<TABLE WIDTH=\"100%\" BORDER=\"4\">\n";
	print "<TR><TD><H3>AS\&nbsp;$as</H3><TD COLSPAN=3><H3>$NAMES{ $as }</H3>\n";
	open( DETAIL, ">$outdir/AS_$as.html" );
	print DETAIL "<HTML>\n";
	print DETAIL "<TITLE>$local_name - $target - AS $as Peering Details</TITLE>\n";
	print DETAIL "<CENTER>\n";
	print DETAIL "<H3>$local_name - $target - AS $as Peering Details</H3>\n";
	print DETAIL "<TABLE WIDTH=\"100%\" BORDER=\"4\">\n";
	print DETAIL "<TR><TD><H3>AS\&nbsp;$as</H3><TD COLSPAN=3><H3>$NAMES{ $as }</H3>\n";
	
	foreach my $peer (sort { $a <=> $b } (keys %{ $AS{$as} }) )
	{
		my @networks = (sort { $a <=> $b } (keys %{ $AS{$as}{$peer}{'routes'} } ));
		print "<TR><TD> \n";
		print "<TR><TD> \n";
		print "<TR><TD> \n";
		print "<TR><TD COLSPAN=4><H4><A NAME=\"$peer\">Peer: $peer</A></H4>\n";
		print "<TR><TD><B>State:<TD>$AS{$as}{$peer}{'state'}</B><TD><B>Uptime:</B><TD>$AS{$as}{$peer}{'uptime'}\n";

		print "<TR><TD><B>Routes:</B><TD COLSPAN=1>" . scalar(@networks). " <TD COLSPAN=2> <A HREF=\"AS_$as.html#$peer\">(view details)</A>\n";
		if ( -f "$outdir/pngs/${peer}.png" )
		{
			print "<TR><TD COLSPAN=4><IMG SRC=\"pngs/$peer.png\">\n";
		}

		print COUNTS "$now $peer " . scalar(@networks) . "\n";

		print DETAIL "<TR><TD> \n";
		print DETAIL "<TR><TD> \n";
		print DETAIL "<TR><TD> \n";
		print DETAIL "<TR><TD COLSPAN=4><H4><A NAME=\"$peer\">Peer: $peer</A></H4>\n";
		print DETAIL "<TR><TD><B>State:<TD>$AS{$as}{$peer}{'state'}</B><TD><B>Uptime:</B><TD>$AS{$as}{$peer}{'uptime'}\n";
		print DETAIL "<TR><TD><B>BGP Version:</B><TD>$AS{$as}{$peer}{'version'}<TD><B>Local Address:</B><TD>$AS{$as}{$peer}{'laddr'}\n";
		print DETAIL "<TR><TD><B>Updates TX:</B><TD>$AS{$as}{$peer}{'InUpdates'}<TD><B>Updates RX:</B><TD>$AS{$as}{$peer}{'OutUpdates'}\n";
		print DETAIL "<TR><TD><B>Messages TX:</B><TD>$AS{$as}{$peer}{'InMessages'}<TD><B>Messages RX:</B><TD>$AS{$as}{$peer}{'OutMessages'}\n";
		print DETAIL "<TR><TD><B>Hold Time:</B><TD>$AS{$as}{$peer}{'ActiveHoldTime'}<TD><B>Keep Alive:</B><TD>$AS{$as}{$peer}{'ActiveKeepAlive'}\n";
		print DETAIL "<TR><TD><B>Last RX'd Update:</B><TD COLSPAN=3>$AS{$as}{$peer}{'TimeSinceLastUpdate'}\&nbsp;ago\n";

		print DETAIL "<TR><TD><B><U>Routes:</U></B><TD>" . scalar(@networks). "<TD COLSPAN=\"2\"><SMALL>* indicates a unicast best route</SMALL>\n";


		while (my $net = shift(@networks))
		{
			
			print DETAIL "<TR>\n";
			my $net2 = shift(@networks);
			my $net3 = shift(@networks);
			my $net4 = shift(@networks);
			foreach my $network ($net, $net2, $net3, $net4)
			{
				if (  $AS{$as}{$peer}{'routes'}{ $network }{ 'best' } eq "true" )
				{
					print DETAIL "<TD ALIGN=\"RIGHT\">*$network\n";
				}
				else
				{
					print DETAIL "<TD ALIGN=\"RIGHT\">&nbsp;$network\n";
				}
			}
		}
		print DETAIL "<TR><TD COLSPAN=4><B><U>Route History:</U></B>\n";
		if ( -f "$outdir/pngs/${peer}.png" )
		{
			print DETAIL "<TR><TD COLSPAN=4><IMG SRC=\"pngs/$peer.png\">\n";
			print DETAIL "<TR><TD COLSPAN=4><IMG SRC=\"pngs/$peer.weekly.png\">\n";
			print DETAIL "<TR><TD COLSPAN=4><IMG SRC=\"pngs/$peer.monthly.png\">\n";
			print DETAIL "<TR><TD COLSPAN=4><IMG SRC=\"pngs/$peer.yearly.png\">\n";
		}
	}

	print DETAIL "</TABLE>\n";
	print DETAIL "<BR>\n";
	print DETAIL "This page last updated: " . localtime();
	print DETAIL "<BR>\n";

	print DETAIL "</CENTER>\n";
	print DETAIL "</HTML>\n";
	close(DETAIL);
	print "</TABLE>\n";

}

print "</TABLE>\n";
print "<BR>\n";
print "This page last updated: " . localtime();
print "<BR>\n";
print "</CENTER>\n";
print "</HTML>\n";
close(MAIN);
close(COUNTS);



sub time_to_text
{
	my ($time) = (@_);
	my $days = $time / (24 * 60 * 60 );
	$days =~ s/\..*$//g;
	$time = $time - ($days * 24 * 60 * 60 );
	my $hours = $time / (60 * 60 );
	$hours =~ s/\..*$//g;
	$time = $time - ($hours * 60 * 60 );
	my $minutes = $time / (60 );
	$minutes =~ s/\..*$//g;
	$time = $time - ($minutes * 60 );
	my $result = sprintf( "%2d days %2d:%02d:%02d", $days, $hours, $minutes, $time );
	return $result;


}

sub ip_to_int
{
	my ($ip) = (@_);
	my (@p) = split( /\./, $ip );
	my $int;
	$int = $int + ( $p[0] * 256 * 256 * 256 );
	$int = $int + ( $p[1] * 256 * 256 );
	$int = $int + ( $p[2] * 256 );
	$int = $int + ( $p[3] );
	return $int;

}
