#! /bin/sh

PKGDIR="$1"
OUTDIR="$2"
TARGET="$3"

PNGDIR=$OUTDIR/$TARGET/pngs
RRDDIR=$PKGDIR/rrds/$TARGET
TMPDIR=$PKGDIR/tmp/$TARGET

test -d $PNGDIR || mkdir -p $PNGDIR
test -d $RDDDIR || mkdir -p $RDDDIR
test -d $TMPDIR || mkdir -p $TMPDIR


TMP=$TMPDIR/rrd_files.tmp
rm -f $TMP

SRC=$TMPDIR/route_counts.txt
test -z "$4" || SRC="$1"

# create/update rrd files
cat $SRC | while read time peer count; do
	RRD="$RRDDIR/${peer}.rrd"
	PNG="$PNGDIR/${peer}"

	# check if the rrd exists, if not create it
	test -f $RRD || rrdtool create -s 300 -b 1108200000 $RRD DS:RouteCount:GAUGE:1800:U:U RRA:LAST:.9:1:2016 RRA:LAST:.9:6:4380
	rrdtool update $RRD "$time:$count" 1>/dev/null 2>&1

	#stash the current count plus some
	#so we can tell rrd ot pad out the top of the graph
	MAX=`expr $count + 2`

	# store info for graph building pass
	echo "$RRD $PNG $MAX" >> $TMP
done

# find the unix epoch time for one week ago
NOW=`date +%s`
GOBACK=`expr 60 \* 60 \* 24 \* 2`
TWO_DAY=`expr $NOW - $GOBACK + 1`
GOBACK=`expr 60 \* 60 \* 24 \* 7`
WEEKLY=`expr $NOW - $GOBACK + 1`
GOBACK=`expr 60 \* 60 \* 24 \* 30`
MONTHLY=`expr $NOW - $GOBACK + 1`
GOBACK=`expr 60 \* 60 \* 24 \* 365`
YEARLY=`expr $NOW - $GOBACK + 1`

# redraw graphs
cat $TMP | sort -u | while read RRD PNG MAX; do
	rrdtool graph $PNG.png --width 500 --start $TWO_DAY -a PNG --lower-limit 0 --upper-limit $MAX -v "Advertised Routes" DEF:routes=$RRD:RouteCount:LAST LINE2:routes#0000FF: 2>&1 1>/dev/null
	rrdtool graph $PNG.weekly.png --width 500 --start $WEEKLY -a PNG --lower-limit 0 --upper-limit $MAX -v "Advertised Routes" DEF:routes=$RRD:RouteCount:LAST LINE2:routes#0000FF: 2>&1 1>/dev/null
	rrdtool graph $PNG.monthly.png --width 500 --start $MONTHLY -a PNG --lower-limit 0 --upper-limit $MAX -v "Advertised Routes" DEF:routes=$RRD:RouteCount:LAST LINE2:routes#0000FF: 2>&1 1>/dev/null
	rrdtool graph $PNG.yearly.png --width 500 --start $YEARLY -a PNG --lower-limit 0 --upper-limit $MAX -v "Advertised Routes" DEF:routes=$RRD:RouteCount:LAST LINE2:routes#0000FF: 2>&1 1>/dev/null
done

rm -f $TMP
